@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -40,8 +40,8 @@ if TYPE_CHECKING:
 # Descriptions of each of the possible config sections.
 # (We use OrderedDict to make the order in which sections are declared in this
 # file be the same order as the sections appear with `streamlit config show`)
-_section_descriptions: OrderedDict[str, str] = OrderedDict(  # ty: ignore
-    _test="Special test section just used for unit tests."  # ty: ignore
+_section_descriptions: OrderedDict[str, str] = OrderedDict(
+    _test="Special test section just used for unit tests."
 )
 
 # Ensures that we don't try to get or set config options when config.toml files
@@ -62,6 +62,20 @@ _config_options: dict[str, ConfigOption] | None = None
 # resolve config and secret files relative to the main script:
 _main_script_path: str | None = None
 
+# Stores the server mode for metrics tracking.
+# Possible values:
+# - "tornado": Traditional Tornado server
+# - "starlette-managed": Starlette server via server.useStarlette config
+# - "starlette-app": st.App started via streamlit run
+# - "asgi-server": st.App with external ASGI server (uvicorn, gunicorn, etc.)
+# - "asgi-mounted": st.App mounted on another ASGI framework (FastAPI, Starlette)
+_server_mode: (
+    Literal[
+        "tornado", "starlette-managed", "starlette-app", "asgi-server", "asgi-mounted"
+    ]
+    | None
+) = None
+
 # Indicates that a config option was defined by the user.
 _USER_DEFINED: Final = "<user defined>"
 
@@ -1003,6 +1017,16 @@ _create_option(
     visibility="hidden",
 )
 
+_create_option(
+    "server.useStarlette",
+    description="""
+        Enable the experimental Starlette-based server implementation instead of
+        Tornado. This is an experimental feature and may be removed in the future.
+    """,
+    default_val=False,
+    type_=bool,
+)
+
 # Config Section: Browser #
 
 _create_section("browser", "Configuration of non-UI browser options.")
@@ -2632,7 +2656,7 @@ def get_config_options(
         # Short-circuit if config files were parsed while we were waiting on
         # the lock.
         if _config_options and not force_reparse:
-            return _config_options  # ty: ignore[invalid-return-type]
+            return _config_options
 
         old_options = _config_options
         _config_options = copy.deepcopy(_config_options_template)
