@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -27,6 +27,7 @@ import streamlit
 from streamlit.elements.lib.layout_utils import LayoutConfig, validate_width
 from streamlit.proto.DocString_pb2 import DocString as DocStringProto
 from streamlit.proto.DocString_pb2 import Member as MemberProto
+from streamlit.runtime.caching.cache_utils import CachedFunc
 from streamlit.runtime.metrics_util import gather_metrics
 from streamlit.runtime.scriptrunner.script_runner import (
     __file__ as SCRIPTRUNNER_FILENAME,  # noqa: N812
@@ -220,6 +221,7 @@ def _get_docstring(obj: object) -> str | None:
     # If that's the case here, use the type's docstring.
     # For objects where type is "type" we do not print the docs (e.g. int).
     # We also do not print the docs for functions and methods if the docstring is empty.
+    # We treat CachedFunc objects in the same way as functions.
     if doc_string is None:
         obj_type = type(obj)
 
@@ -228,6 +230,7 @@ def _get_docstring(obj: object) -> str | None:
             and obj_type is not types.ModuleType
             and not inspect.isfunction(obj)
             and not inspect.ismethod(obj)
+            and obj_type is not CachedFunc
         ):
             doc_string = inspect.getdoc(obj_type)
 
