@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -49,6 +49,9 @@ from streamlit.errors import (
 from streamlit.file_util import get_main_script_directory, normalize_path_join
 from streamlit.navigation.page import StreamlitPage
 from streamlit.proto.Button_pb2 import Button as ButtonProto
+from streamlit.proto.ButtonLikeIconPosition_pb2 import (
+    ButtonLikeIconPosition as ProtoButtonLikeIconPosition,
+)
 from streamlit.proto.DownloadButton_pb2 import DownloadButton as DownloadButtonProto
 from streamlit.proto.LinkButton_pb2 import LinkButton as LinkButtonProto
 from streamlit.proto.PageLink_pb2 import PageLink as PageLinkProto
@@ -86,6 +89,36 @@ DownloadButtonDataType: TypeAlias = (
     | Callable[[], str | bytes | TextIO | BinaryIO | io.RawIOBase]
 )
 
+IconPosition: TypeAlias = Literal["left", "right"]
+
+_DEFAULT_ICON_POSITION: Final[IconPosition] = "left"
+_VALID_ICON_POSITIONS: Final[tuple[IconPosition, ...]] = ("left", "right")
+
+
+def _normalize_icon_position(
+    icon_position: IconPosition | str | None, command: str
+) -> IconPosition:
+    if icon_position is None:
+        return _DEFAULT_ICON_POSITION
+
+    if icon_position not in _VALID_ICON_POSITIONS:
+        raise StreamlitAPIException(
+            f'The icon_position argument to {command} must be "left" or "right". \n'
+            f'The argument passed was "{icon_position}".'
+        )
+
+    return cast("IconPosition", icon_position)
+
+
+def _icon_position_to_proto(
+    icon_position: IconPosition,
+) -> ProtoButtonLikeIconPosition.ValueType:
+    return (
+        ProtoButtonLikeIconPosition.RIGHT
+        if icon_position == "right"
+        else ProtoButtonLikeIconPosition.LEFT
+    )
+
 
 @dataclass
 class ButtonSerde:
@@ -109,6 +142,7 @@ class ButtonMixin:
         *,  # keyword-only arguments:
         type: Literal["primary", "secondary", "tertiary"] = "secondary",
         icon: str | None = None,
+        icon_position: IconPosition = "left",
         disabled: bool = False,
         use_container_width: bool | None = None,
         width: Width = "content",
@@ -188,6 +222,10 @@ class ButtonMixin:
 
             - ``"spinner"``: Displays a spinner as an icon.
 
+        icon_position : "left" or "right"
+            The position of the icon relative to the button label. This
+            defaults to ``"left"``.
+
         disabled : bool
             An optional boolean that disables the button if set to ``True``.
             The default is ``False``.
@@ -324,6 +362,8 @@ class ButtonMixin:
                 f'\nThe argument passed was "{type}".'
             )
 
+        normalized_icon_position = _normalize_icon_position(icon_position, "st.button")
+
         return self.dg._button(
             label,
             key,
@@ -335,6 +375,7 @@ class ButtonMixin:
             disabled=disabled,
             type=type,
             icon=icon,
+            icon_position=normalized_icon_position,
             ctx=ctx,
             width=width,
             shortcut=shortcut,
@@ -355,6 +396,7 @@ class ButtonMixin:
         *,  # keyword-only arguments:
         type: Literal["primary", "secondary", "tertiary"] = "secondary",
         icon: str | None = None,
+        icon_position: IconPosition = "left",
         disabled: bool = False,
         use_container_width: bool | None = None,
         width: Width = "content",
@@ -496,6 +538,10 @@ class ButtonMixin:
 
             - ``"spinner"``: Displays a spinner as an icon.
 
+        icon_position : "left" or "right"
+            The position of the icon relative to the button label. This
+            defaults to ``"left"``.
+
         disabled : bool
             An optional boolean that disables the download button if set to
             ``True``. The default is ``False``.
@@ -696,6 +742,10 @@ class ButtonMixin:
                 f'The argument passed was "{type}".'
             )
 
+        normalized_icon_position = _normalize_icon_position(
+            icon_position, "st.download_button"
+        )
+
         return self._download_button(
             label=label,
             data=data,
@@ -708,6 +758,7 @@ class ButtonMixin:
             kwargs=kwargs,
             type=type,
             icon=icon,
+            icon_position=normalized_icon_position,
             disabled=disabled,
             ctx=ctx,
             width=width,
@@ -723,6 +774,7 @@ class ButtonMixin:
         help: str | None = None,
         type: Literal["primary", "secondary", "tertiary"] = "secondary",
         icon: str | None = None,
+        icon_position: IconPosition = "left",
         disabled: bool = False,
         use_container_width: bool | None = None,
         width: Width = "content",
@@ -794,6 +846,10 @@ class ButtonMixin:
 
             - ``"spinner"``: Displays a spinner as an icon.
 
+        icon_position : "left" or "right"
+            The position of the icon relative to the button label. This
+            defaults to ``"left"``.
+
         disabled : bool
             An optional boolean that disables the link button if set to
             ``True``. The default is ``False``.
@@ -865,6 +921,10 @@ class ButtonMixin:
                 f'\nThe argument passed was "{type}".'
             )
 
+        normalized_icon_position = _normalize_icon_position(
+            icon_position, "st.link_button"
+        )
+
         if use_container_width is not None:
             width = "stretch" if use_container_width else "content"
 
@@ -875,6 +935,7 @@ class ButtonMixin:
             disabled=disabled,
             type=type,
             icon=icon,
+            icon_position=normalized_icon_position,
             width=width,
             shortcut=shortcut,
         )
@@ -886,6 +947,7 @@ class ButtonMixin:
         *,
         label: str | None = None,
         icon: str | None = None,
+        icon_position: IconPosition = "left",
         help: str | None = None,
         disabled: bool = False,
         use_container_width: bool | None = None,
@@ -928,7 +990,7 @@ class ButtonMixin:
             .. _st.markdown: https://docs.streamlit.io/develop/api-reference/text/st.markdown
 
         icon : str or None
-            An optional emoji or icon to display next to the button label. If
+            An optional emoji or icon to display next to the link label. If
             ``icon`` is ``None`` (default), the icon is inferred from the
             ``StreamlitPage`` object or no icon is displayed. If ``icon`` is a
             string, the following options are valid:
@@ -947,6 +1009,10 @@ class ButtonMixin:
 
             - ``"spinner"``: Displays a spinner as an icon.
 
+        icon_position : "left" or "right"
+            The position of the icon relative to the link label. This
+            defaults to ``"left"``.
+
         help : str or None
             A tooltip that gets displayed when the link is hovered over. If
             this is ``None`` (default), no tooltip is displayed.
@@ -1056,10 +1122,15 @@ class ButtonMixin:
             # Sidebar page links should always be stretch width.
             width = "stretch"
 
+        normalized_icon_position = _normalize_icon_position(
+            icon_position, "st.page_link"
+        )
+
         return self._page_link(
             page=page,
             label=label,
             icon=icon,
+            icon_position=normalized_icon_position,
             help=help,
             disabled=disabled,
             width=width,
@@ -1080,6 +1151,7 @@ class ButtonMixin:
         *,  # keyword-only arguments:
         type: Literal["primary", "secondary", "tertiary"] = "secondary",
         icon: str | None = None,
+        icon_position: IconPosition = "left",
         disabled: bool = False,
         ctx: ScriptRunContext | None = None,
         width: Width = "content",
@@ -1117,6 +1189,7 @@ class ButtonMixin:
             help=help,
             type=type,
             width=width,
+            icon_position=icon_position,
             shortcut=normalized_shortcut,
         )
 
@@ -1140,6 +1213,7 @@ class ButtonMixin:
 
         if icon is not None:
             download_button_proto.icon = validate_icon_or_emoji(icon)
+        download_button_proto.icon_position = _icon_position_to_proto(icon_position)
 
         if on_click == "ignore":
             download_button_proto.ignore_rerun = True
@@ -1177,6 +1251,7 @@ class ButtonMixin:
         *,  # keyword-only arguments:
         type: Literal["primary", "secondary", "tertiary"] = "secondary",
         icon: str | None = None,
+        icon_position: IconPosition = "left",
         disabled: bool = False,
         width: Width = "content",
         shortcut: str | None = None,
@@ -1213,6 +1288,7 @@ class ButtonMixin:
 
         if icon is not None:
             link_button_proto.icon = validate_icon_or_emoji(icon)
+        link_button_proto.icon_position = _icon_position_to_proto(icon_position)
 
         if normalized_shortcut is not None:
             link_button_proto.shortcut = normalized_shortcut
@@ -1229,6 +1305,7 @@ class ButtonMixin:
         *,  # keyword-only arguments:
         label: str | None = None,
         icon: str | None = None,
+        icon_position: IconPosition = "left",
         help: str | None = None,
         disabled: bool = False,
         width: Width = "content",
@@ -1240,6 +1317,9 @@ class ButtonMixin:
 
         validate_width(width, allow_content=True)
 
+        # Set icon_position early so it's set even in early return paths
+        page_link_proto.icon_position = _icon_position_to_proto(icon_position)
+
         ctx = get_script_run_ctx()
         if not ctx:
             layout_config = LayoutConfig(width=width)
@@ -1329,6 +1409,7 @@ class ButtonMixin:
         *,  # keyword-only arguments:
         type: Literal["primary", "secondary", "tertiary"] = "secondary",
         icon: str | None = None,
+        icon_position: IconPosition = "left",
         disabled: bool = False,
         ctx: ScriptRunContext | None = None,
         width: Width = "content",
@@ -1362,6 +1443,7 @@ class ButtonMixin:
             is_form_submitter=is_form_submitter,
             type=type,
             width=width,
+            icon_position=icon_position,
             shortcut=normalized_shortcut,
         )
 
@@ -1394,6 +1476,7 @@ class ButtonMixin:
 
         if icon is not None:
             button_proto.icon = validate_icon_or_emoji(icon)
+        button_proto.icon_position = _icon_position_to_proto(icon_position)
 
         if normalized_shortcut is not None:
             button_proto.shortcut = normalized_shortcut
