@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -56,6 +56,7 @@ if TYPE_CHECKING:
     from streamlit.proto.BackMsg_pb2 import BackMsg, DeferredFileRequest
     from streamlit.runtime.script_data import ScriptData
     from streamlit.runtime.scriptrunner.script_cache import ScriptCache
+    from streamlit.runtime.scriptrunner_utils.script_run_context import UserInfoType
     from streamlit.runtime.state import SessionState
     from streamlit.runtime.uploaded_file_manager import UploadedFileManager
     from streamlit.source_util import PageHash, PageInfo
@@ -92,7 +93,7 @@ class AppSession:
         uploaded_file_manager: UploadedFileManager,
         script_cache: ScriptCache,
         message_enqueued_callback: Callable[[], None] | None,
-        user_info: dict[str, str | bool | None],
+        user_info: UserInfoType,
         session_id_override: str | None = None,
     ) -> None:
         """Initialize the AppSession.
@@ -220,6 +221,15 @@ class AppSession:
         self._stop_config_listener = None
         self._stop_pages_listener = None
 
+    def clear_session_caches(self) -> None:
+        """Clears session-level caches for this session.
+
+        This should be called when a session is disconnected or shut down, since this
+        ensures memory is freed up and resource release hooks are called.
+        """
+        caching.clear_session_data_cache(self.id)
+        caching.clear_session_resource_cache(self.id)
+
     def flush_browser_queue(self) -> list[ForwardMsg]:
         """Clear the forward message queue and return the messages it contained.
 
@@ -263,6 +273,9 @@ class AppSession:
             # generally already done so by the time we get here.
             self.disconnect_file_watchers()
 
+            # Clear any session caches. This ensures shutdown hooks are called.
+            self.clear_session_caches()
+
     def _enqueue_forward_msg(self, msg: ForwardMsg) -> None:
         """Enqueue a new ForwardMsg to our browser queue.
 
@@ -690,9 +703,10 @@ class AppSession:
                 )
 
             if self._state == AppSessionState.SHUTDOWN_REQUESTED:
-                # Only clear media files if the script is done running AND the
-                # session is actually shutting down.
+                # Only clear media files and session caches if the script is done
+                # running AND the session is actually shutting down.
                 runtime.get_instance().media_file_mgr.clear_session_refs(self.id)
+                self.clear_session_caches()
 
             self._client_state = client_state
             self._scriptrunner = None
