@@ -1,4 +1,4 @@
-# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
+# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2026)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@ from __future__ import annotations
 
 import json
 import pickle
-from collections.abc import Iterator, KeysView, Mapping, MutableMapping
+from collections.abc import Iterator, KeysView, Mapping, MutableMapping, Sequence
 from copy import deepcopy
 from dataclasses import dataclass, field, replace
 from typing import (
@@ -46,7 +46,12 @@ from streamlit.runtime.state.common import (
 )
 from streamlit.runtime.state.presentation import apply_presenter
 from streamlit.runtime.state.query_params import QueryParams
-from streamlit.runtime.stats import CacheStat, CacheStatsProvider, group_stats
+from streamlit.runtime.stats import (
+    CACHE_MEMORY_FAMILY,
+    CacheStat,
+    StatsProvider,
+    group_cache_stats,
+)
 
 if TYPE_CHECKING:
     from streamlit.runtime.session_manager import SessionManager
@@ -934,12 +939,17 @@ class SessionState:
         else:
             return True
 
-    def get_stats(self) -> list[CacheStat]:
+    def get_stats(
+        self, _family_names: Sequence[str] | None = None
+    ) -> dict[str, list[CacheStat]]:
         # Lazy-load vendored package to prevent import of numpy
         from streamlit.vendor.pympler.asizeof import asizeof
 
         stat = CacheStat("st_session_state", "", asizeof(self))
-        return [stat]
+        # In general, get_stats methods need to be able to return only requested stat
+        # families, but this method only returns a single family, and we're guaranteed
+        # that it was one of those requested if we make it here.
+        return {CACHE_MEMORY_FAMILY: [stat]}
 
     def _check_serializable(self) -> None:
         """Verify that everything added to session state can be serialized.
@@ -992,12 +1002,25 @@ def _is_stale_widget(
 
 
 @dataclass
-class SessionStateStatProvider(CacheStatsProvider):
+class SessionStateStatProvider(StatsProvider):
     _session_mgr: SessionManager
 
-    def get_stats(self) -> list[CacheStat]:
+    @property
+    def stats_families(self) -> Sequence[str]:
+        return (CACHE_MEMORY_FAMILY,)
+
+    def get_stats(
+        self, _family_names: Sequence[str] | None = None
+    ) -> dict[str, list[CacheStat]]:
         stats: list[CacheStat] = []
         for session_info in self._session_mgr.list_active_sessions():
             session_state = session_info.session.session_state
-            stats.extend(session_state.get_stats())
-        return group_stats(stats)
+            session_stats = session_state.get_stats()
+            for family_stats in session_stats.values():
+                stats.extend(family_stats)
+        if not stats:
+            return {}
+        # In general, get_stats methods need to be able to return only requested stat
+        # families, but this method only returns a single family, and we're guaranteed
+        # that it was one of those requested if we make it here.
+        return {CACHE_MEMORY_FAMILY: group_cache_stats(stats)}
